@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -235,11 +235,14 @@ _gnutls_dgram_read (gnutls_session_t session, mbuffer_st **bufel,
   if (ms && *ms > 0)
     {
       gettime(&t2);
-      diff = _dtls_timespec_sub_ms(&t2, &t1);
+      diff = timespec_sub_ms(&t2, &t1);
       if (diff < *ms)
         *ms -= diff;
-      else
-        return gnutls_assert_val(GNUTLS_E_TIMEDOUT);
+      else 
+        {
+          ret = gnutls_assert_val(GNUTLS_E_TIMEDOUT);
+          goto cleanup;
+        }
     }
 
   _gnutls_read_log ("READ: read %d bytes from %p\n", (int) i, fd);
@@ -281,7 +284,10 @@ _gnutls_stream_read (gnutls_session_t session, mbuffer_st **bufel,
         {
           ret = _gnutls_io_check_recv(session, *ms);
           if (ret < 0)
-            return gnutls_assert_val(ret);
+            {
+              gnutls_assert();
+              goto cleanup;
+            }
             
           gettime(&t1);
         }
@@ -308,12 +314,14 @@ _gnutls_stream_read (gnutls_session_t session, mbuffer_st **bufel,
                   goto finish;
                 }
 
-              return errno_to_gerr(err);
+              ret = errno_to_gerr(err);
+              goto cleanup;
             }
           else
             {
               gnutls_assert ();
-              return GNUTLS_E_PULL_ERROR;
+              ret = GNUTLS_E_PULL_ERROR;
+              goto cleanup;
             }
         }
       else
@@ -331,11 +339,14 @@ _gnutls_stream_read (gnutls_session_t session, mbuffer_st **bufel,
       if (ms && *ms > 0)
         {
           gettime(&t2);
-          diff = _dtls_timespec_sub_ms(&t2, &t1);
+          diff = timespec_sub_ms(&t2, &t1);
           if (diff < *ms)
             *ms -= diff;
           else
-            return gnutls_assert_val(GNUTLS_E_TIMEDOUT);
+            {
+              ret = gnutls_assert_val(GNUTLS_E_TIMEDOUT);
+              goto cleanup;
+            }
         }
     }
 
@@ -345,6 +356,10 @@ finish:
                         (int) (size - left), fd);
 
   return (size - left);
+  
+cleanup:
+  _mbuffer_xfree(bufel);
+  return ret;
 }
 
 
@@ -435,7 +450,7 @@ ssize_t
 _gnutls_io_read_buffered (gnutls_session_t session, size_t total,
                           content_type_t recv_type, unsigned int *ms)
 {
-  ssize_t ret = 0;
+  ssize_t ret;
   size_t min;
   mbuffer_st *bufel = NULL;
   size_t recvdata, readsize;
@@ -488,53 +503,35 @@ _gnutls_io_read_buffered (gnutls_session_t session, size_t total,
       /* return immediately if we got an interrupt or eagain
        * error.
        */
-      if (ret < 0 && gnutls_error_is_fatal (ret) == 0)
+      if (ret < 0)
         {
-          _mbuffer_xfree (&bufel);
-          return ret;
+          return gnutls_assert_val(ret);
         }
-    }
 
-  /* copy fresh data to our buffer.
-   */
-  if (ret > 0)
-    {
-      _gnutls_read_log
-        ("RB: Have %d bytes into buffer. Adding %d bytes.\n",
-         (int) session->internals.record_recv_buffer.byte_length, (int) ret);
-      _gnutls_read_log ("RB: Requested %d bytes\n", (int) total);
+      if (ret == 0) /* EOF */
+        return gnutls_assert_val(0);
 
-      _mbuffer_enqueue (&session->internals.record_recv_buffer, bufel);
+	/* copy fresh data to our buffer.
+         */
+        _gnutls_read_log
+          ("RB: Have %d bytes into buffer. Adding %d bytes.\n",
+          (int) session->internals.record_recv_buffer.byte_length, (int) ret);
+        _gnutls_read_log ("RB: Requested %d bytes\n", (int) total);
+
+        _mbuffer_enqueue (&session->internals.record_recv_buffer, bufel);
+
+        if(IS_DTLS(session))
+          ret = MIN(total, session->internals.record_recv_buffer.byte_length);
+        else
+          ret = session->internals.record_recv_buffer.byte_length;
+
+        if ((ret > 0) && ((size_t) ret < total)) /* Short Read */
+          return gnutls_assert_val(GNUTLS_E_AGAIN);
+        else
+          return ret;
     }
   else
-    _mbuffer_xfree (&bufel);
-
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-
-  if (ret == 0)
-    {                           /* EOF */
-      gnutls_assert ();
-      return 0;
-    }
-
-  if(IS_DTLS(session))
-    ret = MIN(total, session->internals.record_recv_buffer.byte_length);
-  else
-    ret = session->internals.record_recv_buffer.byte_length;
-
-  if ((ret > 0) && ((size_t) ret < total))
-    {
-      /* Short Read */
-      return gnutls_assert_val(GNUTLS_E_AGAIN);
-    }
-  else
-    {
-      return ret;
-    }
+    return gnutls_assert_val(0);
 }
 
 /* This function is like write. But it does not return -1 on error.
@@ -851,8 +848,8 @@ parse_handshake_header (gnutls_session_t session, mbuffer_st* bufel,
   /* make the length offset */
   if (hsk->end_offset > 0) hsk->end_offset--;
 
-  _gnutls_handshake_log ("HSK[%p]: %s was received. Length %d[%d], frag offset %d, frag length: %d, sequence: %d\n",
-                         session, _gnutls_handshake2str (hsk->htype),
+  _gnutls_handshake_log ("HSK[%p]: %s (%u) was received. Length %d[%d], frag offset %d, frag length: %d, sequence: %d\n",
+                         session, _gnutls_handshake2str (hsk->htype), (unsigned)hsk->htype,
                          (int) hsk->length, (int)data_size, hsk->start_offset, hsk->end_offset-hsk->start_offset+1, (int)hsk->sequence);
 
   hsk->header_size = handshake_header_size;
@@ -1057,7 +1054,7 @@ _gnutls_parse_record_buffered_msgs (gnutls_session_t session)
           /* this is the rest of a previous message */
           if (session->internals.handshake_recv_buffer_size > 0 && recv_buf[0].length > 0 && remain > 0)
             {
-              if (msg.size <= remain)
+              if ((ssize_t)msg.size <= remain)
                 append = msg.size;
               else
                 append = remain;
