@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -1246,13 +1246,16 @@ _gnutls_handshake_hash_add_sent (gnutls_session_t session,
       if (gnutls_protocol_get_version (session) == GNUTLS_DTLS0_9) 
         {
 	  /* Old DTLS doesn't include the header in the MAC */
-	  if (datalen <= 12) 
+	  if (datalen < 12) 
 	    {
 	      gnutls_assert ();
-	      return GNUTLS_E_INVALID_REQUEST;
+	      return GNUTLS_E_INTERNAL_ERROR;
 	    }
 	  dataptr += 12;
 	  datalen -= 12;
+	  
+	  if (datalen == 0)
+	  	return 0;
         }
 
       ret = _gnutls_buffer_append_data(&session->internals.handshake_hash_buffer,
@@ -2429,6 +2432,10 @@ gnutls_handshake (gnutls_session_t session)
 
   session->security_parameters.epoch_next++;
 
+  ret = _gnutls_ext_after_handshake(session);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
   return 0;
 }
 
@@ -2442,7 +2449,8 @@ gnutls_handshake (gnutls_session_t session)
  * timeout.
  *
  * Note that in order for the timeout to be enforced
- * gnutls_transport_set_pull_timeout_function() must be set.
+ * gnutls_transport_set_pull_timeout_function() must be set
+ * (it is set by default in most systems).
  *
  **/
 void
@@ -2489,10 +2497,14 @@ static int run_verify_callback(gnutls_session_t session, unsigned int side)
   if (type != GNUTLS_CRD_CERTIFICATE)
     return 0;
 
-  if (cred != NULL && cred->verify_callback != NULL)
+  if (cred != NULL && cred->verify_callback != NULL && 
+  	(session->security_parameters.entity == GNUTLS_CLIENT || 
+  		session->internals.send_cert_req != GNUTLS_CERT_IGNORE))
     {
       ret = cred->verify_callback (session);
-      if (ret != 0)
+      if (ret < -1)
+        return ret;
+      else if (ret != 0)
         return GNUTLS_E_CERTIFICATE_ERROR;
     }
   
@@ -3035,6 +3047,7 @@ _gnutls_handshake_server (gnutls_session_t session)
     default:
       break;
     }
+   
 
   return 0;
 }
