@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -107,6 +107,7 @@ typedef struct
 
 /* The maximum digest size of hash algorithms. 
  */
+#define MAX_FILENAME 512
 #define MAX_HASH_SIZE 64
 #define MAX_CIPHER_BLOCK_SIZE 16
 #define MAX_CIPHER_KEY_SIZE 32
@@ -166,6 +167,12 @@ typedef enum transport_t
   GNUTLS_DGRAM
 } transport_t;
 
+typedef enum record_flush_t
+{
+  RECORD_FLUSH = 0,
+  RECORD_CORKED,
+} record_flush_t;
+
 /* the maximum size of encrypted packets */
 #define IS_DTLS(session) (session->internals.transport == GNUTLS_DGRAM)
 
@@ -175,7 +182,8 @@ typedef enum transport_t
 #define RECORD_HEADER_SIZE(session) (IS_DTLS(session) ? DTLS_RECORD_HEADER_SIZE : TLS_RECORD_HEADER_SIZE)
 #define MAX_RECORD_HEADER_SIZE DTLS_RECORD_HEADER_SIZE
 
-#define MAX_RECORD_SEND_SIZE(session) (IS_DTLS(session)?((size_t)gnutls_dtls_get_data_mtu(session)):(size_t)session->security_parameters.max_record_send_size)
+#define MAX_USER_SEND_SIZE(session) (IS_DTLS(session)?((size_t)gnutls_dtls_get_data_mtu(session)):(size_t)session->security_parameters.max_record_send_size)
+#define MAX_RECORD_SEND_SIZE(session) (IS_DTLS(session)?((size_t)gnutls_dtls_get_mtu(session)):(size_t)session->security_parameters.max_record_send_size+MAX_RECORD_OVERHEAD)
 #define MAX_RECORD_RECV_SIZE(session) ((size_t)session->security_parameters.max_record_recv_size)
 #define MAX_PAD_SIZE 255
 #define EXTRA_COMP_SIZE 2048
@@ -265,6 +273,7 @@ typedef enum extensions_t
   GNUTLS_EXTENSION_SRTP = 14,
   GNUTLS_EXTENSION_HEARTBEAT = 15,
   GNUTLS_EXTENSION_SESSION_TICKET = 35,
+  GNUTLS_EXTENSION_NEW_RECORD_PADDING = 48015,  /* aka: 0xbeaf */
   GNUTLS_EXTENSION_SAFE_RENEGOTIATION = 65281   /* aka: 0xff01 */
 } extensions_t;
 
@@ -507,6 +516,9 @@ typedef struct
   uint8_t session_id[TLS_MAX_SESSION_ID_SIZE];
   uint8_t session_id_size;
   time_t timestamp;
+  
+  /* if non-zero the new record padding is used */
+  unsigned int new_record_padding;
 
   /* The send size is the one requested by the programmer.
    * The recv size is the one negotiated with the peer.
@@ -560,6 +572,7 @@ struct record_parameters_st
 
   /* for DTLS */
   uint64_t record_sw[DTLS_RECORD_WINDOW_SIZE];
+  unsigned int record_sw_head_idx;
   unsigned int record_sw_size;
 
   record_state_st read;
@@ -600,8 +613,9 @@ struct gnutls_priority_st
 
   /* to disable record padding */
   unsigned int no_extensions:1;
-  unsigned int no_padding:1;
   unsigned int allow_large_records:1;
+  unsigned int new_record_padding:1;
+  unsigned int max_empty_records;
   safe_renegotiation_t sr;
   unsigned int ssl3_record_version:1;
   unsigned int server_precedence:1;
@@ -609,10 +623,21 @@ struct gnutls_priority_st
   /* Whether stateless compression will be used */
   unsigned int stateless_compression:1;
   unsigned int additional_verify_flags;
+
+  /* The session's expected security level.
+   * Will be used to determine the minimum DH bits,
+   * (or the acceptable certificate security level).
+   */
+  gnutls_sec_param_t level;
+  unsigned int dh_prime_bits; /* old (deprecated) variable */
 };
 
+/* Allow around 50KB of length-hiding padding
+ * when using legacy padding,
+ * or around 3.2MB when using new padding. */
+#define DEFAULT_MAX_EMPTY_RECORDS 200
+
 #define ENABLE_COMPAT(x) \
-              (x)->no_padding = 1; \
               (x)->allow_large_records = 1; \
               (x)->allow_weak_keys = 1
 
@@ -749,6 +774,12 @@ typedef struct
                                          * send.
                                          */
 
+  record_flush_t record_flush_mode; /* GNUTLS_FLUSH or GNUTLS_CORKED */
+  gnutls_buffer_st record_presend_buffer;/* holds cached data
+                                         * for the gnutls_record_send()
+                                         * function.
+                                         */
+
   int expire_time;              /* after expire_time seconds this session will expire */
   struct mod_auth_st_int *auth_struct;  /* used in handshake packets and KX algorithms */
 
@@ -766,12 +797,6 @@ typedef struct
    */
   int send_cert_req;
 
-  /* bits to use for DHE and DHA 
-   * use _gnutls_dh_get_prime_bits() and gnutls_dh_set_prime_bits() 
-   * to access it.
-   */
-  uint16_t dh_prime_bits;
-
   size_t max_handshake_data_buffer_size;
 
   /* PUSH & PULL functions.
@@ -909,6 +934,7 @@ typedef struct
   
   time_t handshake_endtime; /* end time in seconds */
   unsigned int handshake_timeout_ms; /* timeout in milliseconds */
+  unsigned int record_timeout_ms; /* timeout in milliseconds */
 
   gnutls_buffer_st hb_local_data;
   gnutls_buffer_st hb_remote_data;
@@ -978,5 +1004,8 @@ _gnutls_set_current_version (gnutls_session_t session,
   session->security_parameters.version = version;
 }
 
+unsigned int
+/* returns a-b in ms */
+timespec_sub_ms (struct timespec *a, struct timespec *b);
 
 #endif /* GNUTLS_INT_H */
