@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -646,6 +646,9 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
+
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_NORMAL) == 0)
@@ -656,6 +659,9 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
+
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SECURE256) == 0
@@ -668,6 +674,10 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_secure192);
       func (&priority_cache->supported_ecc, supported_ecc_secure192);
+      
+      /* be conservative for now. Set the bits to correspond to 96-bit level */
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_LEGACY;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SECURE128) == 0
@@ -680,6 +690,10 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_secure128);
       func (&priority_cache->supported_ecc, supported_ecc_secure128);
+
+      /* be conservative for now. Set the bits to correspond to an 72-bit level */
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SUITEB128) == 0)
@@ -692,6 +706,9 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_suiteb128);
       func (&priority_cache->supported_ecc, supported_ecc_suiteb128);
+
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_HIGH;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SUITEB192) == 0)
@@ -704,6 +721,9 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_suiteb192);
       func (&priority_cache->supported_ecc, supported_ecc_suiteb192);
+
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_ULTRA;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_EXPORT) == 0)
@@ -714,6 +734,9 @@ bulk_rmadd_func *func;
       func (&priority_cache->sign_algo,
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
+
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_EXPORT;
       return 1;
     }
   return 0;
@@ -803,12 +826,18 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
       gnutls_assert ();
       return GNUTLS_E_MEMORY_ERROR;
     }
+  
+  if (err_pos)
+    *err_pos = priorities;
 
   /* for now unsafe renegotiation is default on everyone. To be removed
    * when we make it the default.
    */
   (*priority_cache)->sr = SR_PARTIAL;
   (*priority_cache)->ssl3_record_version = 1;
+  
+  
+  (*priority_cache)->max_empty_records = DEFAULT_MAX_EMPTY_RECORDS;
 
   if (priorities == NULL)
     priorities = LEVEL_NORMAL;
@@ -831,6 +860,7 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
       _set_priority (&(*priority_cache)->cert_type, cert_type_priority_default);
       _set_priority (&(*priority_cache)->sign_algo, sign_priority_default);
       _set_priority (&(*priority_cache)->supported_ecc, supported_ecc_normal);
+      (*priority_cache)->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       i = 0;
     }
   else
@@ -1036,6 +1066,11 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
             {
               (*priority_cache)->server_precedence = 1;
             }
+          else if (strcasecmp (&broken_list[i][1],
+                               "NEW_PADDING") == 0)
+            {
+              (*priority_cache)->new_record_padding = 1;
+            }
           else
             goto error;
         }
