@@ -11,7 +11,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -832,49 +832,36 @@ gnutls_pkcs11_obj_export (gnutls_pkcs11_obj_t obj,
   return 0;
 }
 
+/**
+ * gnutls_pkcs11_obj_export2:
+ * @obj: Holds the object
+ * @out: will contain a certificate PEM or DER encoded
+ *
+ * This function will export the PKCS11 object data.  It is normal for
+ * data to be inaccesible and in that case %GNUTLS_E_INVALID_REQUEST
+ * will be returned.
+ *
+ * The output buffer is allocated using gnutls_malloc().
+ *
+ * If the structure is PEM encoded, it will have a header
+ * of "BEGIN CERTIFICATE".
+ *
+ * Returns: In case of failure a negative error code will be
+ *   returned, and %GNUTLS_E_SUCCESS (0) on success.
+ *
+ * Since: 3.1.3
+ **/
 int
-pkcs11_find_object (struct pkcs11_session_info* sinfo,
-                    struct pin_info_st * pin_info,
-                    ck_object_handle_t * _obj,
-                    struct p11_kit_uri *info, unsigned int flags)
+gnutls_pkcs11_obj_export2 (gnutls_pkcs11_obj_t obj,
+                           gnutls_datum_t *out)
 {
-  int ret;
-  ck_object_handle_t obj;
-  struct ck_attribute *attrs;
-  unsigned long attr_count;
-  unsigned long count;
-  ck_rv_t rv;
-
-  ret = pkcs11_open_session (sinfo, pin_info, info, flags & SESSION_LOGIN);
-  if (ret < 0)
+  if (obj == NULL || obj->raw.data == NULL)
     {
       gnutls_assert ();
-      return ret;
+      return GNUTLS_E_INVALID_REQUEST;
     }
 
-  attrs = p11_kit_uri_get_attributes (info, &attr_count);
-  rv = pkcs11_find_objects_init (sinfo->module, sinfo->pks, attrs, attr_count);
-  if (rv != CKR_OK)
-    {
-      gnutls_assert ();
-      _gnutls_debug_log ("pk11: FindObjectsInit failed.\n");
-      ret = pkcs11_rv_to_err (rv);
-      goto fail;
-    }
-
-  if (pkcs11_find_objects (sinfo->module, sinfo->pks, &obj, 1, &count) == CKR_OK && count == 1)
-    {
-      *_obj = obj;
-      pkcs11_find_objects_final (sinfo);
-      return 0;
-    }
-
-  ret = GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
-  pkcs11_find_objects_final (sinfo);
-fail:
-  pkcs11_close_session (sinfo);
-
-  return ret;
+  return _gnutls_set_datum(out, obj->raw.data, obj->raw.size);
 }
 
 int
@@ -1161,7 +1148,7 @@ pkcs11_obj_import (ck_object_class_t class, gnutls_pkcs11_obj_t obj,
   return 0;
 }
 
-static int read_pkcs11_pubkey(struct ck_function_list *module,
+int pkcs11_read_pubkey(struct ck_function_list *module,
                               ck_session_handle_t pks, ck_object_handle_t obj,
                               ck_key_type_t key_type, gnutls_datum_t * pubkey)
 {
@@ -1337,7 +1324,7 @@ pkcs11_obj_import_pubkey (struct ck_function_list *module,
     {
       crt->pk_algorithm = mech_to_pk(key_type);
 
-      ret = read_pkcs11_pubkey(module, pks, obj, key_type, crt->pubkey);
+      ret = pkcs11_read_pubkey(module, pks, obj, key_type, crt->pubkey);
       if (ret < 0)
         return gnutls_assert_val(ret);
     }
@@ -1504,7 +1491,6 @@ find_obj_url (struct pkcs11_session_info *sinfo,
 
   while (pkcs11_find_objects (sinfo->module, sinfo->pks, &obj, 1, &count) == CKR_OK && count == 1)
     {
-
       a[0].type = CKA_VALUE;
       a[0].value = cert_data;
       a[0].value_len = MAX_CERT_SIZE;
@@ -2221,11 +2207,12 @@ find_objs (struct pkcs11_session_info* sinfo,
            struct token_info *info, struct ck_info *lib_info, void *input)
 {
   struct crt_find_data_st *find_data = input;
-  struct ck_attribute a[4];
+  struct ck_attribute a[6];
   struct ck_attribute *attr;
   ck_object_class_t class = (ck_object_class_t)-1;
   ck_certificate_type_t type = (ck_certificate_type_t)-1;
   unsigned int trusted;
+  unsigned long category;
   ck_rv_t rv;
   ck_object_handle_t obj;
   unsigned long count;
@@ -2275,6 +2262,16 @@ find_objs (struct pkcs11_session_info* sinfo,
         }
     }
 
+  /* Find objects with given class and type */
+  attr = p11_kit_uri_get_attribute (find_data->info, CKA_CLASS);
+  if (attr)
+    {
+      if(attr->value && attr->value_len == sizeof (ck_object_class_t))
+        class = *((ck_object_class_t*)attr->value);
+      if (class == CKO_CERTIFICATE)
+        type = CKC_X_509;
+    }
+
   cert_data = gnutls_malloc (MAX_CERT_SIZE);
   if (cert_data == NULL)
     {
@@ -2321,6 +2318,28 @@ find_objs (struct pkcs11_session_info* sinfo,
       tot_values++;
 
     }
+  else if (find_data->flags == GNUTLS_PKCS11_OBJ_ATTR_CRT_TRUSTED_CA)
+    {
+      class = CKO_CERTIFICATE;
+      type = CKC_X_509;
+      trusted = 1;
+
+      a[tot_values].type = CKA_CLASS;
+      a[tot_values].value = &class;
+      a[tot_values].value_len = sizeof class;
+      tot_values++;
+
+      a[tot_values].type = CKA_TRUSTED;
+      a[tot_values].value = &trusted;
+      a[tot_values].value_len = sizeof trusted;
+      tot_values++;
+
+      category = 2;
+      a[tot_values].type = CKA_CERTIFICATE_CATEGORY;
+      a[tot_values].value = &category;
+      a[tot_values].value_len = sizeof category;
+      tot_values++;
+    }
   else if (find_data->flags == GNUTLS_PKCS11_OBJ_ATTR_PUBKEY)
     {
       class = CKO_PUBLIC_KEY;
@@ -2538,22 +2557,22 @@ gnutls_pkcs11_obj_list_import_url (gnutls_pkcs11_obj_t * p_list,
                                    unsigned int flags)
 {
   int ret;
-  struct crt_find_data_st find_data;
+  struct crt_find_data_st priv;
 
-  memset (&find_data, 0, sizeof (find_data));
+  memset (&priv, 0, sizeof (priv));
 
   /* fill in the find data structure */
-  find_data.p_list = p_list;
-  find_data.n_list = n_list;
-  find_data.flags = attrs;
-  find_data.current = 0;
+  priv.p_list = p_list;
+  priv.n_list = n_list;
+  priv.flags = attrs;
+  priv.current = 0;
 
   if (url == NULL || url[0] == 0)
     {
       url = "pkcs11:";
     }
 
-  ret = pkcs11_url_to_info (url, &find_data.info);
+  ret = pkcs11_url_to_info (url, &priv.info);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -2561,10 +2580,10 @@ gnutls_pkcs11_obj_list_import_url (gnutls_pkcs11_obj_t * p_list,
     }
 
   ret =
-    _pkcs11_traverse_tokens (find_objs, &find_data, find_data.info,
+    _pkcs11_traverse_tokens (find_objs, &priv, priv.info,
                              NULL,
                              pkcs11_obj_flags_to_int (flags));
-  p11_kit_uri_free (find_data.info);
+  p11_kit_uri_free (priv.info);
 
   if (ret < 0)
     {
@@ -2599,7 +2618,7 @@ gnutls_pkcs11_obj_list_import_url2 (gnutls_pkcs11_obj_t ** p_list,
                                    gnutls_pkcs11_obj_attr_t attrs,
                                    unsigned int flags)
 {
-unsigned int init = 1024;
+unsigned int init = 128;
 int ret;
 
   *p_list = gnutls_malloc(sizeof(gnutls_pkcs11_obj_t)*init);
