@@ -6,7 +6,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -128,12 +128,56 @@ gnutls_pkcs11_privkey_get_info (gnutls_pkcs11_privkey_t pkey,
   return pkcs11_get_info (pkey->info, itype, output, output_size);
 }
 
+static int
+find_object (struct pkcs11_session_info* sinfo,
+                    struct pin_info_st * pin_info,
+                    ck_object_handle_t * _obj,
+                    struct p11_kit_uri *info, unsigned int flags)
+{
+  int ret;
+  ck_object_handle_t obj;
+  struct ck_attribute *attrs;
+  unsigned long attr_count;
+  unsigned long count;
+  ck_rv_t rv;
+
+  ret = pkcs11_open_session (sinfo, pin_info, info, flags & SESSION_LOGIN);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      return ret;
+    }
+
+  attrs = p11_kit_uri_get_attributes (info, &attr_count);
+  rv = pkcs11_find_objects_init (sinfo->module, sinfo->pks, attrs, attr_count);
+  if (rv != CKR_OK)
+    {
+      gnutls_assert ();
+      _gnutls_debug_log ("pk11: FindObjectsInit failed.\n");
+      ret = pkcs11_rv_to_err (rv);
+      goto fail;
+    }
+
+  if (pkcs11_find_objects (sinfo->module, sinfo->pks, &obj, 1, &count) == CKR_OK && count == 1)
+    {
+      *_obj = obj;
+      pkcs11_find_objects_final (sinfo);
+      return 0;
+    }
+
+  ret = GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
+  pkcs11_find_objects_final (sinfo);
+fail:
+  pkcs11_close_session (sinfo);
+
+  return ret;
+}
 
 #define FIND_OBJECT(sinfo, pin_info, obj, key) \
 	do { \
 		int retries = 0; \
 		int rret; \
-		ret = pkcs11_find_object (sinfo, pin_info, &obj, key->info, \
+		ret = find_object (sinfo, pin_info, &obj, key->info, \
 		                          SESSION_LOGIN); \
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) { \
 			if (_gnutls_token_func) \
@@ -500,6 +544,37 @@ int
 gnutls_pkcs11_privkey_generate (const char* url, gnutls_pk_algorithm_t pk, 
                                 unsigned int bits, const char* label, 
                                 unsigned int flags)
+{
+  return gnutls_pkcs11_privkey_generate2( url, pk, bits, label, 0, NULL, flags);
+}
+
+/**
+ * gnutls_pkcs11_privkey_generate2:
+ * @url: a token URL
+ * @pk: the public key algorithm
+ * @bits: the security bits
+ * @label: a label
+ * @fmt: the format of output params. PEM or DER.
+ * @pubkey: will hold the public key (may be %NULL)
+ * @flags: should be zero
+ *
+ * This function will generate a private key in the specified
+ * by the @url token. The private key will be generate within
+ * the token and will not be exportable. This function will
+ * store the DER-encoded public key in the SubjectPublicKeyInfo format 
+ * in @pubkey. The @pubkey should be deinitialized using gnutls_free().
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 3.1.5
+ **/
+int
+gnutls_pkcs11_privkey_generate2 (const char* url, gnutls_pk_algorithm_t pk, 
+                                unsigned int bits, const char* label, 
+                                gnutls_x509_crt_fmt_t fmt, 
+                                gnutls_datum_t * pubkey,
+                                unsigned int flags)
 {
   int ret;
   const ck_bool_t tval = 1;
@@ -512,6 +587,8 @@ gnutls_pkcs11_privkey_generate (const char* url, gnutls_pk_algorithm_t pk,
   unsigned long _bits = bits;
   int a_val, p_val;
   struct ck_mechanism mech;
+  gnutls_pubkey_t pkey = NULL;
+  gnutls_pkcs11_obj_t obj = NULL;
 
   memset(&sinfo, 0, sizeof(sinfo));
 
@@ -664,9 +741,54 @@ gnutls_pkcs11_privkey_generate (const char* url, gnutls_pk_algorithm_t pk,
       ret = pkcs11_rv_to_err (rv);
       goto cleanup;
     }
+  
+  /* extract the public key */
+  if (pubkey)
+    {
+      ret = gnutls_pubkey_init(&pkey);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
+
+      ret = gnutls_pkcs11_obj_init(&obj);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
+
+      obj->pk_algorithm = pk;
+      obj->type = GNUTLS_PKCS11_OBJ_PUBKEY;
+      ret = pkcs11_read_pubkey(sinfo.module, sinfo.pks, pub, mech.mechanism, obj->pubkey);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
+      
+      ret = gnutls_pubkey_import_pkcs11 (pkey, obj, 0);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
+
+      ret = gnutls_pubkey_export2 (pkey, fmt, pubkey);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
+    }
     
 
 cleanup:
+  if (obj != NULL)
+    gnutls_pkcs11_obj_deinit(obj);
+  if (pkey != NULL)
+    gnutls_pubkey_deinit(pkey);
   if (sinfo.pks != 0)
     pkcs11_close_session (&sinfo);
 
