@@ -5,10 +5,10 @@
  *
  * This file is part of libdane.
  *
- * libdane is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
+ * The libdane library is free software; you can redistribute it
+ * and/or modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -204,6 +204,25 @@ void dane_state_deinit(dane_state_t s)
 	free(s);
 }
 
+/**
+ * dane_state_set_dlv_file:
+ * @s: The structure to be deinitialized
+ * @file: The file holding the DLV keys.
+ *
+ * This function will set a file with trusted keys 
+ * for DLV  (DNSSEC  Lookaside  Validation).
+ *
+ **/
+int dane_state_set_dlv_file(dane_state_t s, const char* file)
+{
+int ret;
+  
+  ret = ub_ctx_set_option(s->ctx, (char*)"dlv-anchor-file:", (void*)file);
+  if (ret != 0)
+    return gnutls_assert_val(DANE_E_FILE_ERROR);
+  
+  return 0;
+}
 
 /**
  * dane_query_deinit:
@@ -274,7 +293,7 @@ int dane_query_tlsa(dane_state_t s, dane_query_t *r, const char* host, const cha
 	
 	(*r)->data_entries = i;
 
-	if (!(*r)->result->secure) {
+	if (!(s->flags & DANE_F_INSECURE) && !(*r)->result->secure) {
 		if ((*r)->result->bogus)
 			ret = gnutls_assert_val(DANE_E_INVALID_DNSSEC_SIG);
 		else
